/***************************************************************************** 
*  interrupts.c
*
*  Gergely Zaruba
*  July 2007
*
*
*****************************************************************************/

#include "interrupts.h"


/************************************************************************
 UART0 Receive Interrupt Handler
 ***********************************************************************/ 
interrupt (UART1RX_VECTOR) wakeup SerialReceiveISR(void)
{
    char c;    
    // read receive buffer.  this should auto reset interrupt flag
    c = U1RXBUF;
    // place into receive queue
    serialin_insert_char(c);
    //P2OUT ^= 0x04;
}

/************************************************************************
 UART0 Transmission Done Interrupt Handler
 ***********************************************************************/ 
interrupt (UART1TX_VECTOR) SerialTransmitISR(void)
{
  //this interrupt hits every time a character has been sent
  //on the serial out
    char c;

    if (!serialout_get_head(&c))
      {
        // nothing to transmit, reset interrupt flag and let go
        IFG2 &= ~UTXIFG1;
      }
    else    
      {
        // load buffer
        U1TXBUF = c;
        // interrupt flag should be auto reset
      }
    //    P2OUT ^= 0x10;
}


/************************************************************************
 I2C Interrupt Handler
 ***********************************************************************/ 
interrupt (UART0TX_VECTOR) I2CISR(void)
{
  unsigned char a;
  a=I2CIV;
  //  P2OUT ^= a; 
  switch( a ) //reading this will clear interrupts 
                  //except for RXRDY and TXRFY
  {
  case 2:            // ALIFG: arbritration lost
                     // as in this application there is a single
                     // master this should never happen however see I2C13 errata
    //if slaves are fast enough this is not needed
    /* U0CTL &= ~I2CEN; // disable I2C  see errata I2C11
    I2CIFG = 0;
    I2C_error = true;    
    I2C_block = false; */
    break;

  case 4:          // NACKIFG: no acknowledgement is received from slave
    I2C_error = true; 
    I2C_block = false;
    break;  

  case 6:            // OAIFG: n/a
    break;  
    
  case 8:          // ARDYIFG: release the blocking of the I2C tranmitter/receiver
    I2C_block = false;
    break;  

  case 10:         // RXRDYIFG: n/a
    I2C_in_buffer[I2C_bfr_ptr]=I2CDRB;  //reading the i2c data will reset RXRDYIFG
    I2C_bfr_ptr ++;
    break;  

  case 12:         // TXRDYIFG: n/a
    I2CDRB = I2C_out_buffer[I2C_bfr_ptr]; //writing to the data will clear TXRDYIFG
    I2C_bfr_ptr ++;
    break;  

  case 14:            // GCIFG: n/a
    break;  

  case 16:   break ; // STTIFG: DISABLED
 }
  //P2OUT ^= 0x40;
}



/************************************************************************
 SYSTEM TIMER A Interrupt Handlers
 ***********************************************************************/ 
interrupt (TIMERA0_VECTOR) SystemTimeISR0(void)
{
  //  for generating a 50ms interrupt
  TACCR0 += 50000;
  fifty_milli_second = true;
}

interrupt (TIMERA1_VECTOR) SystemTimeISR1(void)
{
  //this interrupt hits when timerA counter overflows, thus every 0x10000 
  //microseconds; it increments system_time so we do not have to worry 
  //about 16 bit resul;ution of timer A

  // resolution
  if(TACTL & TAIFG)
  {
    system_time+=0x10000;
    TACTL &= ~TAIFG;
  }
}

