/**************************************************************************** 
*  serialout.c
*
*  Gergely Zaruba
*  July 2007
*
****************************************************************************/
#include "serialout.h"


/************************************************************************
 insert a character into the serial transmission buffer
 ***********************************************************************/ 
char critical serialout_insert_code(unsigned char c )
{
  // check if buffer is full
  if ( ((outbuf_head + 1)%SERIAL_OUTPUTBUFFER_SIZE) == outbuf_tail )
    {
      P2OUT ^= 0x10;
      return false;
    }
    
   /*if   ((outbuf_tail == 0 ) && (outbuf_head == SERIAL_OUTPUTBUFFER_SIZE -1))
    {
      P2OUT ^= 0x20;
      return false;
      }*/
  
  serialout_buffer[outbuf_head] = c;
  outbuf_head++;
  outbuf_head %= SERIAL_OUTPUTBUFFER_SIZE;

  // set transmit flag, but only if transmitter empty
  if (U1TCTL & TXEPT)
    IFG2 |= UTXIFG1;
  
  return true;
}

char critical serialout_insert_char( unsigned char c )
{
  char retval;
  unsigned char a;
  a = c & 0x0F;
  retval = serialout_insert_code(HammingCode[a]);
  if(!retval)
    return retval;
  a = (c>>4)&0x0F;
  retval = serialout_insert_code(HammingCode[a]);
  return retval;
}


/************************************************************************
 remove the first character from the serial transmission buffer
 ***********************************************************************/ 
char critical serialout_get_head( char * c )
{
  if(outbuf_head == outbuf_tail)
    return false;

  *c = serialout_buffer[outbuf_tail];
  
  outbuf_tail++;
  outbuf_tail %= SERIAL_OUTPUTBUFFER_SIZE;
  
  return true;
}    


/************************************************************************
 place null terminated string into buffer
 ***********************************************************************/ 
char critical serialout_insert_string( char* str )
{
  // disable interrupts
    while( *str != 0 )
      {
	if(!serialout_insert_char( *str ))
	  return false;
        str++;
      }	
    return true;
}

/************************************************************************
 place null terminated text into buffer 
 ***********************************************************************/ 
char critical serialout_insert_text( char* str )
{
  // disable interrupts
  if(!serialout_insert_code(START_FRAME))
    return false;
  if(!serialout_insert_char(RESPONSE_COMMENT))
    return false;
  while( *str != 0 )
    {
      if(!serialout_insert_char( *str ))
	return false;
      str++;
    }	
  if(!serialout_insert_code(END_FRAME))
    return false;
  return true;
}


/************************************************************************
 place START FRAME and END FRAME into buffer
 ***********************************************************************/ 
char critical serialout_insert_startframe()
{
  // disable interrupts
  return(serialout_insert_code(START_FRAME));
}

char critical serialout_insert_endframe()
{
  // disable interrupts
  return(serialout_insert_code(END_FRAME));
}

/************************************************************************
 place null terminated text start into buffer 
 ***********************************************************************/ 
char critical serialout_insert_texth( char* str )
{
  // disable interrupts
  if(!serialout_insert_code(START_FRAME))
    return false;
  while( *str != 0 )
    {
      if(!serialout_insert_char( *str ))
	return false;
      str++;
    }	
  return true;
}

/************************************************************************
 place null terminated text end into buffer 
 ***********************************************************************/ 
char critical serialout_insert_textt( char* str )
{
  // disable interrupts
  while( *str != 0 )
    {
      if(!serialout_insert_char( *str ))
	return false;
      str++;
    }	
  if(!serialout_insert_code(END_FRAME))
    return false;
  return true;
}


/***************************************************************************
*  outbuffer_empty
*
* is the output buffer empty?
* 
***************************************************************************/
char serialout_buffer_empty()
{
  if(outbuf_head == outbuf_tail)
    return true;
  else
    return false;
}

/***************************************************************************
*  SendError
*
* send error 
* 
***************************************************************************/
char critical serialout_SendError(unsigned char my_error)
{
  //disable interrupts
  if(!serialout_insert_code(START_FRAME))
    return false;
  if(!serialout_insert_char(RESPONSE_ERROR))
    return false;
  if(!serialout_insert_char(my_error))
    return false;
  if(!serialout_insert_code(END_FRAME))
    return false;
  return true;
}

/***************************************************************************
*  SendError
*
* send error 
* 
***************************************************************************/
char critical serialout_SendOK(unsigned char my_error)
{
  //disable interrupts
  if(!serialout_insert_code(START_FRAME))
    return false;
  if(!serialout_insert_char(RESPONSE_OK))
    return false;
  if(!serialout_insert_char(my_error))
    return false;
  if(!serialout_insert_code(END_FRAME))
    return false;
  return true;
}
